@@ -259,10 +259,10 @@
      "in" "infix" "infixl" "infixr" "instance" "let" "module" "newtype" "of"
      "then" "type" "where")
     (python-mode ;; https://docs.python.org/3/reference/lexical_analysis.html#keywords
-     "False" "None" "True" "and" "as" "assert" "break" "class" "continue" "def"
-     "del" "elif" "else" "except" "exec" "finally" "for" "from" "global" "if"
-     "import" "in" "is" "lambda" "nonlocal" "not" "or" "pass" "print" "raise"
-     "return" "try" "while" "with" "yield")
+     "False" "None" "True" "and" "as" "assert" "break" "case" "class" "continue"
+     "def" "del" "elif" "else" "except" "exec" "finally" "for" "from" "global"
+     "if" "import" "in" "is" "lambda" "match" "nonlocal" "not" "or" "pass"
+     "print" "raise" "return" "try" "while" "with" "yield")
     (ruby-mode
      "BEGIN" "END" "alias" "and" "begin" "break" "case" "class" "def" "defined?"
      "do" "else" "elsif" "end" "ensure" "false" "for" "if" "in" "module" "next"
@@ -407,7 +407,7 @@ If INTERACTIVE is nil the function acts like a capf."
     (when-let (keywords (cape--keyword-list))
       (let ((bounds (cape--bounds 'symbol)))
         `(,(car bounds) ,(cdr bounds)
-          ,(cape--table-with-properties keywords :category 'cape-keyword)
+          ,(cape--properties-table keywords :category 'cape-keyword)
           ,@cape--keyword-properties)))))
 
 (provide 'cape-keyword)
